<?php

namespace Ems\Services;

use Ems\Repository\{
    CertisignRepository,
    OrganizacaoRepository
};
use Ems\Util\Base64Util;
use Ems\Util\ExpectedException;
use Ems\Util\RequestUtils;
use GuzzleHttp\Client;
use Util\Environment;
use Util\NovoCodigo;
use Ems\Services\{
    ParamOrgService,
    ParceiroNegocioService
};

class CertisignService {
    /** @var Environment */
    protected $environment;

    /** @var CertisignRepository */
    protected $certisignRepository;

    /** @var ParamOrgService */
    protected $paramOrgService;

    /** @var Client */
    protected $http;

    protected $accountInfo = null;

    /** @var string */
    protected $certisignUrl = null;

    /** @var ParceiroNegocioService */
    protected $parceiroNegocioService;

    /** @var OrganizacaoRepository */
    protected $organizacaoRepository;

    public function __construct(
        Environment         $environment,
        CertisignRepository $certisignRepository,
        ParamOrgService     $paramOrgService,
        string              $certisignUrl,
        ParceiroNegocioService  $parceiroNegocioService,
        OrganizacaoRepository   $organizacaoRepository
    ) {
        $this->environment          = $environment;
        $this->certisignRepository  = $certisignRepository;
        $this->paramOrgService      = $paramOrgService;
        $this->certisignUrl         = $certisignUrl;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->organizacaoRepository    = $organizacaoRepository;
    }

    public function getAccountInfo() {
        if ($this->accountInfo !== null) {
            return $this->accountInfo;
        }

        $paramOrg = $this->paramOrgService->get();

        $paramsAreSet = isset($paramOrg['DSTOKENAELETRON'])
            && isset($paramOrg['NMREMAELETRON'])
            && isset($paramOrg['DSEMAILAELETRON'])
            && isset($paramOrg['NRCPFREMAELETRON']);

        if (!$paramsAreSet) {
            return null;
        }

        $accountInfo = [
            'token' => $paramOrg['DSTOKENAELETRON'],
            'name' => $paramOrg['NMREMAELETRON'],
            'email' => $paramOrg['DSEMAILAELETRON'],
            'cpf' => $paramOrg['NRCPFREMAELETRON'],
        ];

        return $accountInfo;
    }

    public function getUri(
        string $route
    ): string {
        $uri = $this->certisignUrl . $route;
        return $uri;
    }

    public function upload(
        string $fileName,
        string $base64
    ): string {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri('/document/upload');

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'fileName' => $fileName,
            'bytes' => $base64,
        ];

        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $uploadId = $responseBody['uploadId'] ?? null;

        if (!$uploadId) {
            throw new ExpectedException('Não foi possível fazer o upload do arquivo no Certisign!');
        }

        return $uploadId;
    }

    public function delete(
        string $id
    ) {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri("document/delete/$id");

        $headers = [
            'Token' => $accountInfo['token'],
        ];

        $body = null;

        $responseBody = RequestUtils::request($uri, $body, 'DELETE', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if (!$code) {
            return;
        }

        throw new ExpectedException("(Certisign) $message");
    }

    public function create(
        array $options
    ) {
        $accountInfo = $this->getAccountInfo();

        $documentName = $options['documentName'] ?? null;
        $uploadId = $options['uploadId'] ?? null;
        $uploadName = $options['uploadId'] ?? null;
        $signers =  $options['signers'] ?? [];
        $eSigners =  $options['eSigners'] ?? [];

        $uri = $this->getUri('document/create');

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'document' => [
                'name' => $documentName,
                'upload' => [
                    'id' => $uploadId,
                    'name' => $uploadName,
                ],
            ],
            'sender' => [
                'name' => $accountInfo['name'],
                'email' => $accountInfo['email'],
                'individualIdentificationCode' => $accountInfo['cpf'],
            ],
            'signers' => $signers,
            'electronicSigners' => $eSigners,
        ];

        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if (!$code) {
            return $responseBody;
        }

        throw new ExpectedException("(Certisign) $message");
    }

    public function validateSignatures(
        string $documentKey
    ) {
        $accountInfo = $this->getAccountInfo();

        $uri = $this->getUri("document/ValidateSignatures/?key=$documentKey");

        $headers = [
            'Token' => $accountInfo['token'],
        ];

        $body = null;

        $responseBody = RequestUtils::request($uri, $body, 'GET', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if ($code) {
            throw new ExpectedException("(Certisign) $message");
        }

        return $responseBody;
    }

    public function callback(
        array $body
    ) {
        $this->setNrOrgFromHeader();

        $action = $body['action'];

        if ($action === 'SIGNATURE-ELETRONIC') {
            $this->documentOnSignature($body);
        }

        if ($action === 'SIGNATURE-NOK') {
            $this->documentOnRejection($body);
        }
    }

    private function setNrOrgFromHeader() {
        $headers = getallheaders();

        $clientSecret = $headers['authentication'] ?? null;

        if ($clientSecret === null) {
            var_dump('Token de autenticação não está presente!', $headers);die;
        }

        $organizacao = $this->organizacaoRepository->getOrganizacaoByClientSecret($clientSecret, true);
        $nrOrg = (int) $organizacao['NRORG'];

        $this->environment->setNrOrg($nrOrg);
    }

    public function documentOnSignature(
        array $body
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $documentId = $body['documentId'];
        $identifier = $body['identifier'];

        $parcNegocio = $this->parceiroNegocioService->getParceiroNegocioByNrInscricao(
            $identifier
        );

        if (!$parcNegocio) {
            throw new ExpectedException("Não foi encontrado o parceiro de negócio com número de inscrição $identifier");
        }

        $nrParcNegocio = $parcNegocio['NRPARCNEGOCIO'];

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
            'P_IDDOCUMENTO' => $documentId,
        ];

        $this->certisignRepository->setDocumentAsSigned($params);
    }

    public function documentOnRejection(
        array $body
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $documentId = $body['documentId'];
        $identifier = $body['identifier'];
        $message = $body['message'];

        $parcNegocio = $this->parceiroNegocioService->getParceiroNegocioByNrInscricao(
            $identifier
        );

        if (!$parcNegocio) {
            throw new ExpectedException("Não foi encontrado o parceiro de negócio com número de inscrição $identifier");
        }

        $nrParcNegocio = $parcNegocio['NRINSCRICAOPARC'];

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARCNEGOCIO' => $nrParcNegocio,
            'P_IDDOCUMENTO' => $documentId,
            'P_DSREJEICAO' => $message,
        ];

        $this->certisignRepository->setDocumentAsRejected($params);
    }
}